@@ -1,4 +1,4 @@
-/* $Id: cut.c,v 1.60 2003/04/19 19:34:05 astyanax Exp $ */
+/* $Id: cut.c,v 1.62 2003/08/05 19:31:12 jordi Exp $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -46,7 +46,7 @@ filestruct *get_cutbottom(void)
 void add_to_cutbuffer(filestruct *inptr)
 {
 #ifdef DEBUG
-    fprintf(stderr, _("add_to_cutbuffer() called with inptr->data = %s\n"),
+    fprintf(stderr, "add_to_cutbuffer() called with inptr->data = %s\n",
 	    inptr->data);
 #endif
 
@@ -210,7 +210,7 @@ int do_cut_text(void)
 	concatenate_cut = 0;
 #endif
 #ifdef DEBUG
-	fprintf(stderr, _("Blew away cutbuffer =)\n"));
+	fprintf(stderr, "Blew away cutbuffer =)\n");
 #endif
     }
 
@@ -361,7 +361,7 @@ int do_uncut_text(void)
 	    size_t buf_len = strlen(cutbuffer->data);
 	    size_t cur_len = strlen(current->data);
 
-	    current->data = nrealloc(current->data, cur_len + buf_len + 1);
+	    current->data = charealloc(current->data, cur_len + buf_len + 1);
 	    memmove(current->data + current_x + buf_len,
 			current->data + current_x, cur_len - current_x + 1);
 	    strncpy(current->data + current_x, cutbuffer->data, buf_len);
